@import"https://fonts.googleapis.com/css2?family=Inter&display=swap";.control-menu-container{position:fixed}.control-menu{background-color:#f8f9fc;position:fixed;width:350px;padding-block:30px;padding-inline:20px;height:calc(100vh - 70px);overflow-y:auto}.menu-item{width:100%;min-height:150px;background-color:#fff;border-radius:5px;padding:20px 10px 30px;box-shadow:0 4px 6px #00000026;margin-bottom:20px}.menu-item__header{display:flex;align-items:center;margin-bottom:8px}.menu-item__content{display:flex;margin-top:20px;flex-direction:column;gap:10px}.menu-item__content label{padding-left:10px}.menu-item h3{font-weight:400;font-size:18px;margin-bottom:5px}.menu-item img{width:20px;margin-right:10px}.error{color:red;font-size:15px;margin-block:10px}.add-input::placeholder{color:#505050;border:none}.add-button{position:absolute;top:10px;right:0;cursor:pointer;display:flex;align-items:center;padding:5px;border:none;background-color:#fff;box-shadow:0 1px 2px #00000040;border-radius:100%}.upload-form{margin-bottom:15px}.file-item{display:flex;align-items:center;justify-content:space-between;margin-bottom:10px;color:#505050;font-size:15px}.remove-button{cursor:pointer;border:none;background-color:transparent;padding:0;margin:0}.remove-button svg:hover{color:red}.resources-list{padding-top:30px;padding-left:35px;margin-left:350px}.resources-list h1{font-family:Inter;font-weight:200;font-size:38px;line-height:46px;color:#000}.resources-list__table{display:flex;flex-wrap:wrap;gap:15px}.resources-list__table__card{width:210px;padding:20px 15px 25px;box-shadow:1px 2px 4px #00000040}.resource-name{color:#000;font-family:Inter;font-size:18px;margin-bottom:5px;display:-webkit-box;-webkit-line-clamp:1;-webkit-box-orient:vertical;overflow:hidden}.resources-image{width:40px}.resource-type{color:#a3a3a3;font-family:Inter;font-size:16px;font-weight:400}.resource-type_container{display:flex;align-items:center;gap:5px;margin-block:10px}.resource-status{margin-bottom:15px}.resources-list__table__card button{border:none;border-radius:5px;background:#4285f4;color:#fff;padding:6px 25px;cursor:pointer}.resource-details{padding:30px;display:flex;flex-direction:column;gap:10px}.resource-details__item{display:flex;align-items:center;margin-bottom:10px}.resource-details__item__key{color:#a3a3a3;font-family:Inter;font-weight:400;margin-right:10px;width:220px}.header-container{display:flex;justify-content:space-between;align-items:center;text-align:center;margin-bottom:30px}.header-text{margin:0;display:flex;align-items:center}.button-container{display:flex;margin-right:40px}.button-container button{background-color:transparent;border:1px solid #333;padding:10px;margin-right:10px;display:flex;align-items:center;cursor:pointer;transition:background-color .2s}button.active{background-color:#ccc}.icon{margin-right:5px;font-size:20px;width:20px;height:20px}tr{height:10px}tr td,tr th{height:auto!important;padding:5px!important}.table-button button{border:none;border-radius:5px;background:#4285f4;color:#fff;padding:6px 25px;cursor:pointer}.roles-list{padding-top:30px;padding-left:35px;margin-left:350px}.roles-list h1{font-family:Inter;font-weight:200;font-size:38px;line-height:46px;color:#000;margin-bottom:30px}header{position:sticky;top:0;display:flex;align-items:center;padding:8px 0;border-bottom:2px solid #dadce0;background-color:#fff}header .logo{margin-right:10px;margin-left:30px;width:39px}header p{font-size:20px;font-weight:300}.pages-nav{position:sticky;top:50px;padding:10px 0;border-bottom:2px solid #dadce0;background-color:#fff;width:100%;display:grid;scrollbar-gutter:stable both-edges;grid-template-columns:1fr .8fr 1fr;align-items:center}.links{display:flex}.pages-nav a{padding:0 20px;font-size:16px;text-decoration:none;color:#000}.pages-nav a:hover{color:#3367d6}.pages-nav a.active{color:#3367d6;font-weight:500}.pages-nav a.active:after{content:"";position:relative;top:19px;left:-20px;display:block;width:calc(100% + 40px);height:2px;background-color:#3367d6}.search-bar{background-color:#f4f4f4;border:.5px solid #999999;border-radius:5px;padding:6px 15px;display:flex;justify-content:space-between}#search-input{background-color:#f4f4f4;border:none;font-size:14px;flex-grow:1}#search-input:focus{outline:none}*{margin:0;padding:0;box-sizing:border-box;font-family:Inter,sans-serif}html{scrollbar-gutter:stable}
